unit udmsqldb;

{$mode ObjFPC}{$H+}

//Switches:
//You can disable switches by using a period after the curved
//bracket like this  {.$DEFINE SWITCHNAME}

//delete Databasefile before ZConnection1.Open so every start an new and empty file is created
{$DEFINE DeleteDatabasefileBeforeOpen}

//Import Data automatically on Create. You can use the procedure ImportData to import
//data at any given time
{.$DEFINE ImportDemoDataOnCreate}


interface

uses
  Classes, SysUtils, SQLite3Conn, SQLDBLib, SQLDB, Forms, uguessfile;

const
  {$include demosqlite.Inc}

type

  { Tdmsqldb }

  Tdmsqldb = class(TDataModule)
    SQLConnector1: TSQLConnector;
    SQLDBLibraryLoader1: TSQLDBLibraryLoader;
    SQLQuery1: TSQLQuery;
    SQLScript1: TSQLScript;
    SQLTransaction1: TSQLTransaction;
    procedure DataModuleCreate(Sender: TObject);
  private

  public
    procedure Initialize;
    procedure ImportData(sImportFile: string='');
    procedure OpenConnection;
    procedure CloseConnection;
  end;

var
  dmsqldb: Tdmsqldb;

implementation

uses
  dialogs;
{$R *.lfm}

{ Tdmsqldb }

procedure Tdmsqldb.DataModuleCreate(Sender: TObject);
begin
  Initialize;

  //Import SQL File (demo data, database structure etc)
  {$ifdef ImportDemoDataOnCreate}
    ImportData;
  {$endif}

end;

procedure Tdmsqldb.Initialize;
var
  sLibrary:string;
  sSQLfile:string;
begin
  CloseConnection;                              //in case SQLDBLibraryLoader1 or SQLConnector1 were active in design mode close them
  SQLDBLibraryLoader1.Enabled:=false;           //triggers an internal UnloadLibrary

  sLibrary :='';
  sLibrary := GuessLibraryLocation;             //Search for sqlite Library

  if sLibrary = cUNDEF then exit;               //for security reasons in case the exception fails

  //First: load library
  SQLDBLibraryLoader1.ConnectionType:=cConnectorType; //define connector Type
  SQLDBLibraryLoader1.LibraryName:=sLibrary;
  SQLDBLibraryLoader1.Enabled:=true;            //triggers an internal LoadLibrary
  //SQLDBLibraryLoader1.LoadLibrary;            //fpcsrc\packages\fcl-db\examples\sqlite3loadlib.lpr uses Loadlibrary additionally

  //Second: Try to find the SQLite Database file
  //        If it does not exist SQLite creates the named database file automatically
  {$ifdef DeleteDatabasefileBeforeOpen}
   sSQLFile := GuessSQLDatabaseFile(true);
  {$else}
   sSQLFile := GuessSQLDatabaseFile(false);
  {$endif}

  // only then establish a connection to the database
  SQLConnector1.DatabaseName:=sSQLFile;
  SQLConnector1.ConnectorType:=cConnectorType;

  SQLConnector1.Open;
end;

procedure Tdmsqldb.ImportData(sImportFile: string);
begin
  if sImportfile = '' then begin
     sImportFile := GuessSQLImportFile;
     if sImportFile = cUNDEF then begin //just in case
        showmessage( 'SQL import file not found');
        exit;
     end
  end;
  if not FileExists(sImportFile) then begin
     showmessage('ImportData: File ' + sImportFile +'does not exist');
     exit;
  end;
  try
    SQLScript1.Script.LoadFromFile(sImportFile);
    SQLScript1.ExecuteScript;     //oder.Execute???
  finally
    //
  end;

end;


procedure Tdmsqldb.OpenConnection;
begin
  if not SQLConnector1.Connected then SQLConnector1.Open;
end;

procedure Tdmsqldb.CloseConnection;
begin
  if SQLConnector1.Connected then SQLConnector1.Close;
end;


end.

