unit uguessfile;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils,forms, dialogs;

const
  {$include demosqlite.Inc}

function GuessFile(const sFileName: string; const sWhereToSearch: string;
  var sSearched: string): string;      //tries to find a file in application dirextory and other directories
function GuessLibraryLocation: string; //versucht die SQLite-DLL in Standardverzeichnissen zu suchen und gibt den Pfad+Dateinamen zurück
function GuessSQLImportFile(bAllowOpenDialog:boolean=false):string;    //versucht in standardverzeichnissen das ImportFile für FileDatenbanken (SQLITE etc) zu finden
function GuessSQLDatabaseFile(bDeleteOldFile:boolean=false):string;  //versucht in standardverzeichnissen das Datenfile für FileDatenbanken (SQLITE etc) zu finden
function GetTargetCPUinfo:string;

implementation

function GuessFile(const sFileName: string; const sWhereToSearch: string;
  var sSearched: string): string;
var
  s:string;
  sl:TStringlist;
  i: integer;
begin
  result:=cUNDEF;

  s :=  Application.Location + sFileName;
  if FileExists(s) then
  begin
   result := s;
   exit;
  end;
  sSearched:=sSearched + s + LineEnding;

  sl := TStringList.Create;
  try
    sl.Delimiter:=cDelimiter;
    sl.DelimitedText := sWhereToSearch;
    for i := 0 to sl.Count - 1 do begin
      s :=  Application.Location + sl[i]+ DirectorySeparator + sFileName;
      if FileExists(s) then
      begin
       result := s;
       break;
      end;
      sSearched:=sSearched + s + LineEnding;
    end;
  finally
    sl.Free;
  end;

end;

function GuessLibraryLocation: string;
var
  s: string;
  sSearched:string;
  sPathString:string;
begin
  result:=cUNDEF;
  sSearched:='';


  {$ifdef Unix}
  sPathString:='to define'
  {$else}
  sPathString := 'dll@sql';
  {$endif}

  result:=GuessFile(cSQLdbLibraryName,sPathString,sSearched);

{ Nur für Windows-OS spezielle Suche im System-Verzeichnis ---CAVE bitness

ür Windows gibt es eine Unit WinDirs (in fpc/source/rtl/win) mit Konstanten
CSIDL_PROGRAMS, CSIDL_PERSONAL usw, und den Funktionen GetWindowsSpecialDir(),
GetWindowsSpecialDirUnicode(), die damit aufgerufen werden
können und das entsprechende Spezial-Verzeichnis zurückgeben.

und auch für andere  OS:
https://www.lazarusforum.de/viewtopic.php?p=103860&sid=8adc0c5cdb6ef3589f446620f3d8a2a4#p103860


s :=  Application.Location + 'dll'+ DirectorySeparator + 'standard'+ DirectorySeparator
  + cSQLiteDllName;
if FileExists(s) then
begin
 result := s;
 exit;
end;
sSearched:=sSearched+s+LineEnding;
}


  if result=cUNDEF  then
  raise Exception.Create( LineEnding
                          + 'File: ' + {$INCLUDE %FILE%} + LineEnding
                          + 'Methodname: ' +  {$I %CURRENTROUTINE%} + LineEnding
                          + 'Linenumber: ' + {$INCLUDE %LINE%} + LineEnding +LineEnding
                          + 'tried to find Library ' + cSQLdbLibraryName + ' in '
                          + LineEnding + LineEnding
                          + sSearched
                          + 'but it was not found!' + LineEnding+ LineEnding
                          + 'Please provide ' + cSQLdbLibraryName +' in one of the searchpaths');

end;

function GuessSQLImportFile(bAllowOpenDialog: boolean): string;
var
  s:string;
  sSearchedin:string;
  sPathString:string;
  OpenDialog:TOpendialog;
begin
  result:=cUNDEF;
  sSearchedin:='';

  {$ifdef Unix}
  sPathString:='to define'
  {$else}
  sPathString := 'dll@data';
  {$endif}

  result:=GuessFile(cSQLCustomImportFileName,sPathString,sSearchedin);
  if result=cUNDEF  then
    result:=GuessFile(cSQLStandardImportFileName,sPathString,sSearchedin);


  if (result=cUNDEF) and (bAllowOpenDialog) then begin
    OpenDialog := TOpendialog.Create(nil);
    OpenDialog.DefaultExt:='*.sql';
    try
      if OpenDialog.Execute then
        result := OpenDialog.Filename
      else
        result:=cUNDEF;
    finally
      freeandnil(OpenDialog);
    end;
  end;


  if result=cUNDEF  then
  raise Exception.Create( LineEnding
                          + 'File: ' + {$INCLUDE %FILE%} + LineEnding
                          + 'Methodname: ' +  {$I %CURRENTROUTINE%} + LineEnding
                          + 'Linenumber: ' + {$INCLUDE %LINE%} + LineEnding +LineEnding
                          + 'tried to find SQL import File '
                          + 'either' + cSQLCustomImportFileName + ' or ' + cSQLStandardImportFileName + ' in '
                          + LineEnding + LineEnding
                          + sSearchedin
                          + 'but it was not found!' + LineEnding+ LineEnding
                          + 'Please provide ' + 'either' + cSQLCustomImportFileName + ' or ' + cSQLStandardImportFileName +' in one of the searchpaths');

end;

function GuessSQLDatabaseFile(bDeleteOldFile: boolean): string;
var
  s:string;
  sSearchedin:string;
  sPathString:string;
begin
  result:=cUNDEF;
  sSearchedin:='';

  {$ifdef Unix}
  sPathString:='to define'
  {$else}
  sPathString := 'data@sql';
  {$endif}

  result:=GuessFile(cSQLDatabaseFile,sPathString,sSearchedin);

  if result=cUNDEF  then begin
    showmessage( LineEnding
                + 'File: ' + {$INCLUDE %FILE%} + LineEnding
                + 'Methodname: ' +  {$I %CURRENTROUTINE%} + LineEnding
                + 'Linenumber: ' + {$INCLUDE %LINE%} + LineEnding +LineEnding
                + 'tried to find SQL data File in'
                + LineEnding + LineEnding
                + sSearchedin
                + 'but it was not found!' + LineEnding+ LineEnding
                + 'Please provide '  + cSQLDatabaseFile +' in one of the searchpaths'+ LineEnding+ LineEnding
                +' creating standard database file '+cSQLDatabaseFile+' in application directory instead');

    result:=cSQLDatabaseFile;
  end;

  if bDeleteOldFile then
    if FileExists(result) then DeleteFile(result);

end;


function GetTargetCPUinfo: string;
begin
  result := {$I %FPCTARGETCPU%};

{
if GetTargetInfo = 'i386' then
    ZConnection2.LibraryLocation:=(ExtractFilePath(Application.ExeName)+'32\sqlite3.dll')
else
   ZConnection2.LibraryLocation:=(ExtractFilePath(Application.ExeName)+'64\sqlite3.dll');
end;
}
end;

end.

