unit usqldbruntime;

{$mode ObjFPC}{$H+}

{Switches:
You can disable switches by using a period after the curved
bracket like this  {.$DEFINE SWITCHNAME}
}

{delete Databasefile before ZConnection1.Open so on every application
 start a new and empty file is created}
{$DEFINE DeleteDatabasefileBeforeOpen}

{Import Data automatically on Create. You can use the procedure ImportData to import
data at any given time}
{.$DEFINE ImportDemoDataOnCreate}

{Note: For a demo or sample application it can be advantageous
 that the same initial data is available at every program start.
 In this case activate both switches}

//*********extended settings
{If you want to use a SQL log  enable the switch}
{.$DEFINE UseSQLLOG}

{If you want to use an additional possibility to query
 the database beside other Queries then enable the switch - SQLQuery1 will be created}
{.$DEFINE UseAdditionalQuery}

interface

uses
  Classes, SysUtils, SQLDBLib, SQLDB, Forms, uguessfile, dialogs
  , SQLite3Conn
  //if you want to use other database connections than SQLite you need the corresponding includes
  //sqlite3conn, pqconnection, ibconnection, mssqlconn, oracleconnection,mysql55conn,mysql40conn,mysql51conn,mysql50conn;
  ;

const
  {$include demosqlite.Inc}

//TSQLConnType, TSQLServerType  copied from SQLDBToolsunit.pas therefore they may be unsynchronized
type
  //TSQLConnType = (mysql40,mysql41,mysql50,mysql51,mysql55,mysql56,mysql57,mysql80,postgresql,interbase,odbc,oracle,sqlite3,mssql,sybase);
  TSQLServerType = (ssFirebird, ssInterbase, ssMSSQL, ssMySQL, ssOracle, ssPostgreSQL, ssSQLite, ssSybase, ssUnknown);


type

  { Tsqldbruntime }

  Tsqldbruntime = class(TObject)
    SQLDBLibraryLoader1: TSQLDBLibraryLoader;
    SQLTransaction1: TSQLTransaction;
    {$ifdef UseSQLLOG}
    procedure DoSQLLog(Sender: TSQLConnection; EventType: TDBEventType; const Msg: String);
    {$endif}
  private
    //SQLConnType : TSQLConnType;
    SQLServerType : TSQLServerType;
  public
    SQLConnector1: TSQLConnector;
    {$ifdef UseAdditionalQuery}
    SQLQuery1: TSQLQuery;
    {$endif}
    constructor Create;
    destructor Destroy;
    procedure ConnectToDatabase;
    procedure ImportData(sImportFile:string='');
    procedure OpenConnection;
    procedure CloseConnection;
  end;


implementation

{ Tsqldbruntime }


procedure Tsqldbruntime.ConnectToDatabase;
var
  sLibrary:string;
  sSQLfile:string;
begin
  CloseConnection;                               //in case SQLDBLibraryLoader1 or SQLConnector1 were active

  sLibrary :='';
  sLibrary := GuessLibraryLocation;              //Search for sqlite Library
  if sLibrary = cUNDEF then exit;               //for security reasons in case the exception fails

  //First: load library
  SQLDBLibraryLoader1.Enabled:=false;            //triggers an internal UnloadLibrary
  SQLDBLibraryLoader1.ConnectionType:=cConnectorType;
  SQLDBLibraryLoader1.LibraryName:=sLibrary;
  SQLDBLibraryLoader1.Enabled:=true;            //triggers an internal LoadLibrary

  //Second: Try to find the SQLite Database file
  //        If it does not exist SQLite creates the named database file automatically

  //distinguish file databases versus server based databases
  case SQLServerType of
    ssSQLite,
    ssInterbase,
    ssFirebird  : begin
      {$ifdef DeleteDatabasefileBeforeOpen}
       sSQLFile := GuessSQLDatabaseFile(true);
      {$else}
       sSQLFile := GuessSQLDatabaseFile(false);
      {$endif}
      SQLConnector1.DatabaseName:=sSQLFile;
    end
    else begin
      //feel free to get credentials elsewhere - INI file for example
      SQLConnector1.DatabaseName:=cSQLDatabasename;
      SQLConnector1.UserName:=cSQLuser;
      SQLConnector1.Password:=cSQLpassword;
      SQLConnector1.Hostname:=cSQLhostname;
    end;
  end;

  SQLConnector1.ConnectorType:=cConnectorType;
  SQLConnector1.CharSet:='UTF8';
  SQLConnector1.Options:=[scoApplyUpdatesChecksRowsAffected];

  {$ifdef UseSQLLOG}
  SQLConnector1.OnLog:=@DoSQLLog;
  SQLConnector1.LogEvents:=LogAllEventsExtra;  //either LogAllEvents or LogAllEventsExtra
  {$endif}
  SQLConnector1.Open;

end;

constructor Tsqldbruntime.Create;
begin

  SQLConnector1:= TSQLConnector.Create(nil);
  SQLDBLibraryLoader1:=TSQLDBLibraryLoader.Create(nil);

  SQLTransaction1 := TSQLTransaction.Create(nil);
  //tweak Transaction behaviour
  SQLTransaction1.Options:=[stoUseImplicit];
  //SQLTransaction1.Params.Add('version1');
  //SQLTransaction1.Params.Add('read_committed');
  //SQLTransaction1.Params.Add('isc_tpb_lock_read=PARMTAB');

  //SQLConnector1.Transaction:= SQLTransaction1;

  ConnectToDatabase;

  {$ifdef UseAdditionalQuery}
  SQLQuery1:=TSQLQuery.Create(nil);
  SQLQuery1.Database:=SQLdbConnection;
  {$endif}

  //Import SQL File (demo data, database structure etc)
  {$ifdef ImportDemoDataOnCreate}
    ImportData;
  {$endif}
end;

destructor Tsqldbruntime.Destroy;
begin
   if SQLConnector1.Connected then SQLConnector1.Close();
   if SQLDBLibraryLoader1.Enabled then SQLDBLibraryLoader1.Enabled:=false;

   {$ifdef UseAdditionalQuery}
   if SQLQuery1.Active then SQLQuery1.Close;
   SQLQuery1.free;
   {$endif}

   SQLDBLibraryLoader1.free;
   SQLConnector1.free;
   SQLTransaction1.free;
end;

procedure Tsqldbruntime.ImportData(sImportFile: string);
var
  MyScript:TSQLScript;
begin

  if sImportfile = '' then begin
     sImportFile := GuessSQLImportFile(true);
     if sImportFile = cUNDEF then begin //just in case
        showmessage( 'SQL import file not found');

        //OpenDialog := TOpendialog.Create(nil);
        //OpenDialog.DefaultExt:='*.sql';
        //try
        //  if OpenDialog.Execute then
        //    sImportFile := OpenDialog.Filename
        //  else
        //    sImportFile:=cUNDEF;
        //finally
        //  freeandnil(OpenDialog);
        //end;

     end;
  end;
  if sImportFile=cUNDEF then  exit;

  if not FileExists(sImportFile) then begin
     showmessage('ImportData: File ' + sImportFile +'does not exist');
     exit;
  end;

  MyScript:=TSQLScript.Create(nil);
  try
    MyScript.Transaction:=SQLTransaction1;
    MyScript.Terminator:=';';
    MyScript.DataBase:=SQLConnector1;
    MyScript.CommentsInSQL:=true;
    MyScript.UseCommit:=true;
    MyScript.UseDefines:=true;
    MyScript.UseSetTerm:=true;
    MyScript.Script.LoadFromFile(sImportFile);
    MyScript.Execute;
  finally
    freeandnil(MyScript);
  end;

end;

procedure Tsqldbruntime.OpenConnection;
begin
  if not SQLConnector1.Connected then SQLConnector1.Open;
end;

procedure Tsqldbruntime.CloseConnection;
begin
  if SQLConnector1.Connected then SQLConnector1.Close;
end;

{$ifdef UseSQLLOG}
procedure Tsqldbruntime.DoSQLLog(Sender: TSQLConnection;
  EventType: TDBEventType; const Msg: String);
begin
  //just a simple message on stdError
  //Feel free to enhance it
  Writeln(stderr,'[',EventType,'] : ',Msg);
end;
{$endif}

end.

